/* ************************************************************************

   Copyright:

   License:

   Authors:

************************************************************************ */

qx.Theme.define("callbackery.theme.Theme",
{
  meta :
  {
    color : callbackery.theme.Color,
    decoration : callbackery.theme.Decoration,
    font : callbackery.theme.Font,
    icon : qx.theme.icon.Tango,
    appearance : callbackery.theme.Appearance
  }
});